import os
import tkinter as tk
from tkinter import messagebox
from PIL import Image, ImageDraw, ImageTk

# Constants
DATA_PATH = '/home/mark/Projects/glyph-generator/data2'
OUTPUT_PATH = f'{DATA_PATH}/input/glyphs-unclassified'
GLYPH_SIZE = 16
SCALE = 50  # Display scale for each pixel

# Ensure output directory exists
os.makedirs(OUTPUT_PATH, exist_ok=True)

class GlyphEditor:
    def __init__(self, root):
        self.root = root
        self.mode = 'CREATE'
        self.file_index = 0
        self.files = []

        self.update_title()

        # Canvas for drawing
        self.canvas = tk.Canvas(root, width=GLYPH_SIZE * SCALE, height=GLYPH_SIZE * SCALE, bg='black')
        self.canvas.pack()

        # Shortcut key instructions
        self.info_label = tk.Label(root, text="Shortcuts: c = Create, m = Modify/Edit, s/space = Save, n = Next, p = Previous, d = Delete, g = Toggle Grid, x = Exit, arrows = Shift Pixels", bg="gray", fg="white")
        self.info_label.pack(fill=tk.X)

        # Initialize glyph grid (2D array)
        self.grid = [[0 for _ in range(GLYPH_SIZE)] for _ in range(GLYPH_SIZE)]

        # Grid visibility toggle
        self.grid_visible = False

        # Event bindings
        self.canvas.bind('<B1-Motion>', self.draw_pixel)  # Left mouse button to draw
        self.canvas.bind('<B3-Motion>', self.erase_pixel)  # Right mouse button to erase
        self.root.bind('c', self.set_create_mode)  # Set to Create mode
        self.root.bind('m', self.set_edit_mode)  # Set to Edit mode
        self.root.bind('s', self.save_glyph)  # Save glyph
        self.root.bind('<space>', self.save_glyph)  # Save glyph with spacebar
        self.root.bind('n', self.next_file)  # Next file
        self.root.bind('p', self.previous_file)  # Previous file
        self.root.bind('d', self.delete_file)  # Delete file
        self.root.bind('g', self.toggle_grid)  # Toggle grid visibility
        self.root.bind('x', self.exit_editor)  # Exit editor
        self.root.bind('<Left>', self.shift_left)  # Shift pixels left
        self.root.bind('<Right>', self.shift_right)  # Shift pixels right
        self.root.bind('<Up>', self.shift_up)  # Shift pixels up
        self.root.bind('<Down>', self.shift_down)  # Shift pixels down

        if self.mode == 'EDIT':
            self.load_files()

    def update_title(self):
        """Update the window title to show the current mode and file."""
        if self.mode == 'CREATE':
            index = len([f for f in os.listdir(OUTPUT_PATH) if f.endswith('.jpg')])  # Count only .jpg files  # Predict the next file name
            self.root.title(f"Glyph Editor - Mode: {self.mode} - Next File: M3_{index}.jpg")
        elif self.mode == 'EDIT' and self.files:
            current_file = self.files[self.file_index] if self.files else "None"
            self.root.title(f"Glyph Editor - Mode: {self.mode} - File: {current_file}")
        else:
            self.root.title(f"Glyph Editor - Mode: {self.mode}")

    def set_create_mode(self, event):
        """Switch to Create mode."""
        self.mode = 'CREATE'
        self.clear_grid()
        self.update_title()
        print("Mode: CREATE")

    def set_edit_mode(self, event):
        """Switch to Edit mode."""
        self.mode = 'EDIT'
        self.load_files()
        if self.files:
            self.load_file()
        else:
            messagebox.showinfo("Info", "No files to edit.")
        self.update_title()

    def load_files(self):
        """Load all files for Edit mode in sorted order."""
        self.files = sorted([f for f in os.listdir(OUTPUT_PATH) if f.endswith('.jpg')])

    def draw_pixel(self, event):
        """Draw a pixel (set to white) on the canvas and grid."""
        x, y = event.x // SCALE, event.y // SCALE
        if 0 <= x < GLYPH_SIZE and 0 <= y < GLYPH_SIZE:
            self.grid[y][x] = 1
            self.update_canvas()

    def erase_pixel(self, event):
        """Erase a pixel (set to black) on the canvas and grid."""
        x, y = event.x // SCALE, event.y // SCALE
        if 0 <= x < GLYPH_SIZE and 0 <= y < GLYPH_SIZE:
            self.grid[y][x] = 0
            self.update_canvas()

    def update_canvas(self):
        """Update the canvas to reflect the current grid."""
        self.canvas.delete("all")
        for y in range(GLYPH_SIZE):
            for x in range(GLYPH_SIZE):
                color = 'white' if self.grid[y][x] == 1 else 'black'
                self.canvas.create_rectangle(
                    x * SCALE, y * SCALE, (x + 1) * SCALE, (y + 1) * SCALE, fill=color, outline=color
                )
        if self.grid_visible:
            self.draw_grid()

    def draw_grid(self):
        """Draw the grid lines on the canvas."""
        for i in range(GLYPH_SIZE + 1):
            # Horizontal lines
            self.canvas.create_line(0, i * SCALE, GLYPH_SIZE * SCALE, i * SCALE, fill="red")
            # Vertical lines
            self.canvas.create_line(i * SCALE, 0, i * SCALE, GLYPH_SIZE * SCALE, fill="red")

    def toggle_grid(self, event):
        """Toggle the visibility of the grid lines."""
        self.grid_visible = not self.grid_visible
        self.update_canvas()

    def save_glyph(self, event):
        """Save the current glyph to a file and advance in Edit mode."""
        if self.mode == 'CREATE':
            index = len(os.listdir(OUTPUT_PATH))  # Auto-name based on file count
            output_file = f'{OUTPUT_PATH}/M2_{index}.jpg'
        else:
            output_file = os.path.join(OUTPUT_PATH, self.files[self.file_index])
            self.file_index = (self.file_index + 1) % len(self.files)  # Advance to the next file

        # Create and save the glyph image
        img = Image.new('L', (GLYPH_SIZE, GLYPH_SIZE), color=0)
        draw = ImageDraw.Draw(img)
        for y in range(GLYPH_SIZE):
            for x in range(GLYPH_SIZE):
                if self.grid[y][x] == 1:
                    draw.point((x, y), fill=255)
        img.save(output_file)
        print(f"Saved: {output_file}")

        if self.mode == 'CREATE':
            self.clear_grid()
            self.update_title()
        elif self.mode == 'EDIT':
            self.load_file()

    def load_file(self):
        """Load a glyph from an existing file."""
        file_path = os.path.join(OUTPUT_PATH, self.files[self.file_index])
        img = Image.open(file_path).convert('L')
        self.grid = [[1 if img.getpixel((x, y)) > 128 else 0 for x in range(GLYPH_SIZE)] for y in range(GLYPH_SIZE)]  # type: ignore
        self.update_canvas()
        self.update_title()
        print(f"Loaded: {file_path}")

    def next_file(self, event):
        """Go to the next file in Edit mode."""
        if self.mode == 'EDIT' and self.files:
            self.file_index = (self.file_index + 1) % len(self.files)
            self.load_file()

    def previous_file(self, event):
        """Go to the previous file in Edit mode."""
        if self.mode == 'EDIT' and self.files:
            self.file_index = (self.file_index - 1) % len(self.files)
            self.load_file()

    def delete_file(self, event):
        """Delete the current file and move to the next one."""
        if self.mode == 'EDIT' and self.files:
            file_path = os.path.join(OUTPUT_PATH, self.files[self.file_index])
            os.remove(file_path)
            print(f"Deleted: {file_path}")
            del self.files[self.file_index]

            if self.files:
                self.file_index %= len(self.files)
                self.load_file()
            else:
                print("No more files to edit.")
                self.clear_grid()
                self.update_title()

    def shift_left(self, event):
        """Shift all pixels to the left."""
        for y in range(GLYPH_SIZE):
            self.grid[y] = self.grid[y][1:] + [0]
        self.update_canvas()

    def shift_right(self, event):
        """Shift all pixels to the right."""
        for y in range(GLYPH_SIZE):
            self.grid[y] = [0] + self.grid[y][:-1]
        self.update_canvas()

    def shift_up(self, event):
        """Shift all pixels up."""
        self.grid = self.grid[1:] + [[0] * GLYPH_SIZE]
        self.update_canvas()

    def shift_down(self, event):
        """Shift all pixels down."""
        self.grid = [[0] * GLYPH_SIZE] + self.grid[:-1]
        self.update_canvas()

    def clear_grid(self):
        """Clear the grid for the next glyph."""
        self.grid = [[0 for _ in range(GLYPH_SIZE)] for _ in range(GLYPH_SIZE)]
        self.update_canvas()

    def exit_editor(self, event):
        """Exit the editor."""
        print("Exiting editor.")
        self.root.destroy()

# Main application
if __name__ == '__main__':
    root = tk.Tk()
    app = GlyphEditor(root)
    root.mainloop()
